@@ -583,8 +583,9 @@ error:
 str* get_p_notify_body(str pres_uri, pres_ev_t* event, str* etag,
 		str* contact)
 {
-	db_key_t query_cols[3];
-	db_val_t query_vals[3];
+	db_key_t query_cols[4];
+	db_val_t query_vals[4];
+	db_op_t query_ops[4];
 	db_key_t result_cols[3];
 	db1_res_t *result = NULL;
 	int body_col, etag_col= 0, sender_col;
@@ -634,18 +635,28 @@ str* get_p_notify_body(str pres_uri, pres_ev_t* event, str* etag,
 	query_vals[n_query_cols].type = DB1_STR;
 	query_vals[n_query_cols].nul = 0;
 	query_vals[n_query_cols].val.str_val = uri.host;
+	query_ops[n_query_cols] = OP_EQ;
 	n_query_cols++;
 
 	query_cols[n_query_cols] = &str_username_col;
 	query_vals[n_query_cols].type = DB1_STR;
 	query_vals[n_query_cols].nul = 0;
 	query_vals[n_query_cols].val.str_val = uri.user;
+	query_ops[n_query_cols] = OP_EQ;
 	n_query_cols++;
 
 	query_cols[n_query_cols] = &str_event_col;
 	query_vals[n_query_cols].type = DB1_STR;
 	query_vals[n_query_cols].nul = 0;
 	query_vals[n_query_cols].val.str_val= event->name;
+	query_ops[n_query_cols] = OP_EQ;
+	n_query_cols++;
+
+	query_cols[n_query_cols] = &str_expires_col;
+	query_vals[n_query_cols].type = DB1_INT;
+	query_vals[n_query_cols].nul = 0;
+	query_vals[n_query_cols].val.int_val= (int)time(NULL);
+	query_ops[n_query_cols] = OP_GT;
 	n_query_cols++;
 
 	result_cols[body_col=n_result_cols++] = &str_body_col;
@@ -659,11 +670,11 @@ str* get_p_notify_body(str pres_uri, pres_ev_t* event, str* etag,
 	}
 
 	if(pres_retrieve_order==1) {
-		query_str = str_priority_col;
+		query_str = pres_retrieve_order_by;
 	} else {
 		query_str = str_received_time_col;
 	}
-	if (pa_dbf.query (pa_db, query_cols, 0, query_vals,
+	if (pa_dbf.query (pa_db, query_cols, query_ops, query_vals,
 		 result_cols, n_query_cols, n_result_cols, &query_str ,  &result) < 0) 
 	{
 		LM_ERR("failed to query %.*s table\n", presentity_table.len, presentity_table.s);
@@ -1253,7 +1264,7 @@ error:
 
 int publ_notify(presentity_t* p, str pres_uri, str* body, str* offline_etag, str* rules_doc)
 {
-	str *notify_body = NULL, *aux_body = NULL;
+	str *notify_body = NULL;
 	subs_t* subs_array= NULL, *s= NULL;
 	int ret_code= -1;
 
@@ -1280,22 +1291,13 @@ int publ_notify(presentity_t* p, str pres_uri, str* body, str* offline_etag, str
 	while(s)
 	{
 		s->auth_rules_doc= rules_doc;
-		if (p->event->aux_body_processing) {
-			aux_body = p->event->aux_body_processing(s, notify_body?notify_body:body);
-		}
 
-		if(notify(s, NULL, aux_body?aux_body:(notify_body?notify_body:body), 0)< 0 )
+		if(notify(s, NULL, notify_body?notify_body:body, 0, p->event->aux_body_processing)< 0 )
 		{
 			LM_ERR("Could not send notify for %.*s\n",
 					p->event->name.len, p->event->name.s);
 		}
 
-		if(aux_body!=NULL) {
-			if(aux_body->s)	{
-				p->event->aux_free_body(aux_body->s);
-			}
-			pkg_free(aux_body);
-		}
 		s= s->next;
 	}
 	ret_code= 0;
@@ -1443,7 +1445,7 @@ int query_db_notify(str* pres_uri, pres_ev_t* event, subs_t* watcher_subs )
 				aux_body = event->aux_body_processing(s, notify_body);
 			}
 	
-			if(notify(s, watcher_subs, aux_body?aux_body:notify_body, 0)< 0 )
+			if(notify(s, watcher_subs, aux_body?aux_body:notify_body, 0, 0)< 0 )
 			{
 				LM_ERR("Could not send notify for [event]=%.*s\n",
 						event->name.len, event->name.s);
@@ -1656,8 +1658,11 @@ error:
 }
 
 
-int notify(subs_t* subs, subs_t * watcher_subs,str* n_body,int force_null_body)
+int notify(subs_t* subs, subs_t * watcher_subs,str* n_body,int force_null_body, aux_body_processing_t* aux_body_processing)
 {
+
+	str* aux_body = NULL;
+
 	/* update first in hash table and the send Notify */
 	if(subs->expires!= 0 && subs->status != TERMINATED_STATUS)
 	{
@@ -1694,11 +1699,29 @@ int notify(subs_t* subs, subs_t * watcher_subs,str* n_body,int force_null_body)
 		force_null_body = 1;
 	}
 
-	if(send_notify_request(subs, watcher_subs, n_body, force_null_body)< 0)
+	if (!force_null_body && aux_body_processing)
+	{
+		aux_body = aux_body_processing(subs, n_body);
+	}
+
+	if(send_notify_request(subs, watcher_subs, aux_body?aux_body:n_body, force_null_body)< 0)
 	{
 		LM_ERR("sending Notify not successful\n");
+		if(aux_body!=NULL) {
+			if(aux_body->s) {
+				subs->event->aux_free_body(aux_body->s);
+			}
+			pkg_free(aux_body);
+		}
 		return -1;
 	}
+
+	if(aux_body!=NULL) {
+		if(aux_body->s) {
+			subs->event->aux_free_body(aux_body->s);
+		}
+		pkg_free(aux_body);
+	}
 	return 0;
 }
 
@@ -2126,7 +2149,7 @@ error:
 #define EXTRACT_STRING(strng, chars)\
 			do {\
 			strng.s = (char *) chars;\
-			strng.len = strlen(strng.s);\
+			strng.len = strng.s == NULL ? 0 : strlen(strng.s);\
 			} while(0);
 
 static int unset_watchers_updated_winfo(str *pres_uri)
@@ -2720,7 +2743,7 @@ static int notifier_notify(subs_t *sub, int *updated, int *end_transaction)
 			goto done;
 	}
 
-	if (notify(sub, NULL, nbody, 0) < 0)
+	if (notify(sub, NULL, nbody, 0, 0) < 0)
 	{
 		LM_ERR("could not send notify\n");
 		goto error;
