@@ -63,6 +63,7 @@ MODULE_VERSION
 #define DEF_USE_PUBRURI_AVPS 0
 #define DEF_PUBRURI_CALLER_AVP 0
 #define DEF_PUBRURI_CALLEE_AVP 0
+#define DEF_CALLEE_TRYING 0
 
 
 /* define PUA_DIALOGINFO_DEBUG to activate more verbose 
@@ -91,6 +92,7 @@ int caller_confirmed       = DEF_CALLER_ALWAYS_CONFIRMED;
 int include_req_uri        = DEF_INCLUDE_REQ_URI;
 int send_publish_flag      = DEF_SEND_PUBLISH_FLAG;
 int use_pubruri_avps       = DEF_USE_PUBRURI_AVPS;
+int callee_trying          = DEF_CALLEE_TRYING;
 char * pubruri_caller_avp  = DEF_PUBRURI_CALLER_AVP;
 char * pubruri_callee_avp  = DEF_PUBRURI_CALLEE_AVP;
 
@@ -119,6 +121,7 @@ static param_export_t params[]={
 	{"pubruri_callee_avp",  PARAM_STRING, &pubruri_callee_avp },
 	{"pubruri_caller_dlg_var",  PARAM_STR, &caller_dlg_var },
 	{"pubruri_callee_dlg_var",  PARAM_STR, &callee_dlg_var },
+	{"callee_trying",        INT_PARAM, &callee_trying },
 	{0, 0, 0 }
 };
 
@@ -251,10 +254,15 @@ __dialog_sendpublish(struct dlg_cell *dlg, int type, struct dlg_cb_params *_para
 	str tag = {0,0};
 	str uri = {0,0};
 	str target = {0,0};
+	struct dlginfo_cell *dlginfo = NULL;
 
+	dlginfo = (struct dlginfo_cell*)*_params->param;
 
-	struct dlginfo_cell *dlginfo = (struct dlginfo_cell*)*_params->param;
-
+	if(dlg==NULL || dlginfo==NULL) {
+		LM_WARN("execution with null parameters - type %d, dlg %p, info %p\n",
+				type, dlg, dlginfo);
+		return;
+	}
 	if(include_req_uri) {
 		uri = dlginfo->req_uri;
 	} else {
@@ -599,6 +607,14 @@ __dialog_created(struct dlg_cell *dlg, int type, struct dlg_cb_params *_params)
 			&(dlg->callid), 1, dlginfo->lifetime,
 			0, 0, 0, 0, (send_publish_flag==-1)?1:0);
 
+	if (callee_trying)
+	{
+		dialog_publish_multi("Trying", dlginfo->pubruris_callee,
+				(include_req_uri)?&(dlg->req_uri):&(dlg->to_uri),
+				&(dlg->from_uri),
+				&(dlg->callid), 0, dlginfo->lifetime,
+				0, 0, 0, 0, (send_publish_flag==-1)?1:0);
+	}
 }
 
 	static void
@@ -609,7 +625,11 @@ __dialog_loaded(struct dlg_cell *dlg, int type, struct dlg_cb_params *_params)
 	LM_DBG("INVITE dialog loaded: from=%.*s\n", dlg->from_uri.len, dlg->from_uri.s);
 
 	dlginfo=get_dialog_data(dlg, type);
-	if(dlginfo!=NULL) free_dlginfo_cell(dlginfo);
+	if(dlginfo!=NULL) {
+		LM_DBG("dialog info initialized (from=%.*s)\n",
+				dlg->from_uri.len, dlg->from_uri.s);
+		/* free_dlginfo_cell(dlginfo); */
+	}
 }
 
 
