@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * ratelimit module
  *
  * Copyright (C) 2006 Hendrik Scholz <hscholz@raisdorf.net>
@@ -22,12 +20,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * ---------
- *
- * 2008-01-10 ported from SER project (osas)
- * 2008-01-16 ported enhancements from openims project (osas)
- * 2020-04-28 add sip-router rpc interface (osas)
  */
 
 #include <stdio.h>
@@ -333,14 +325,20 @@ static int get_cpuload(double * load)
 	FILE * f = fopen("/proc/stat", "r");
 	double vload;
 	int ncpu;
+	static int errormsg = 0;
 
 	if (! f) {
-		LM_ERR("could not open /proc/stat\n");
+		/* Only output this error message five times */
+		if (errormsg < 5) {
+			LM_ERR("could not open /proc/stat\n");
+			errormsg++;
+		}
 		return -1;
 	}
 	if (fscanf(f, "cpu  %lld%lld%lld%lld%lld%lld%lld%lld",
 			&n_user, &n_nice, &n_sys, &n_idle, &n_iow, &n_irq, &n_sirq, &n_stl) < 0) {
 		  LM_ERR("could not parse load information\n");
+		  fclose(f);
 		  return -1;
 	}
 	fclose(f);
@@ -361,9 +359,6 @@ static int get_cpuload(double * load)
 
 		vload = ((double)d_idle) / (double)d_total;
 
-		/* divide by numbers of cpu */
-		ncpu = get_num_cpus();
-		vload = vload/ncpu;
 		vload = 1.0 - vload;
 		if(vload<0.0) vload = 0.0;
 		else if (vload>1.0) vload = 1.0;
